# matrix-appservice-kakaotalk - A Matrix-KakaoTalk puppeting bridge.
# Copyright (C) 2022 Tulir Asokan, Andrew Ferrazzutti
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
from mautrix.bridge.commands import HelpSection, command_handler

from .typehint import CommandEvent

SECTION_CONNECTION = HelpSection("Connection management", 15, "")


@command_handler(
    needs_auth=False,
    management_only=True,
    help_section=SECTION_CONNECTION,
    help_text="Mark this room as your bridge notice room",
)
async def set_notice_room(evt: CommandEvent) -> None:
    evt.sender.notice_room = evt.room_id
    await evt.sender.save()
    await evt.reply("This room has been marked as your bridge notice room")


"""
@command_handler(
    needs_auth=True,
    management_only=True,
    help_section=SECTION_CONNECTION,
    help_text="Disconnect from KakaoTalk",
)
async def disconnect(evt: CommandEvent) -> None:
    if not evt.sender.mqtt:
        await evt.reply("You don't have a KakaoTalk MQTT connection")
        return
    evt.sender.mqtt.disconnect()


@command_handler(
    needs_auth=True,
    management_only=True,
    help_section=SECTION_CONNECTION,
    help_text="Connect to KakaoTalk",
    aliases=["reconnect"],
)
async def connect(evt: CommandEvent) -> None:
    if evt.sender.listen_task and not evt.sender.listen_task.done():
        await evt.reply("You already have a KakaoTalk MQTT connection")
        return
    evt.sender.start_listen()
"""


@command_handler(
    needs_auth=True,
    management_only=True,
    help_section=SECTION_CONNECTION,
    help_text="Check if you're logged into KakaoTalk",
)
async def ping(evt: CommandEvent) -> None:
    if not await evt.sender.is_logged_in():
        await evt.reply("You're not logged into KakaoTalk")
        return
    # try:
    own_info = await evt.sender.get_own_info()
    # TODO catch errors
    # except fbchat.PleaseRefresh as e:
    #     await evt.reply(f"{e}\n\nUse `$cmdprefix+sp refresh` refresh the session.")
    #     return
    await evt.reply(f"You're logged in as {own_info.nickname} (user ID {evt.sender.ktid})")

    """
    if not evt.sender.listen_task or evt.sender.listen_task.done():
        await evt.reply("You don't have a KakaoTalk MQTT connection. Use `connect` to connect.")
    elif not evt.sender.is_connected:
        await evt.reply("The KakaoTalk MQTT listener is **disconnected**.")
    else:
        await evt.reply("The KakaoTalk MQTT listener is connected.")
    """


"""
@command_handler(
    needs_auth=True,
    management_only=True,
    help_section=SECTION_CONNECTION,
    help_text="Resync chats and reconnect to MQTT",
)
async def refresh(evt: CommandEvent) -> None:
    await evt.sender.refresh(force_notice=True)
"""


@command_handler(
    needs_auth=True,
    management_only=True,
    help_section=SECTION_CONNECTION,
    help_text="Resync chats",
)
async def sync(evt: CommandEvent) -> None:
    await evt.mark_read()
    await evt.sender.post_login(is_startup=False)
    await evt.reply("Sync complete")
